class App {
  constructor() {
    this._onUpload = this._onUpload.bind(this);
    this._onDownload = this._onDownload.bind(this);

    document.querySelector("#uploadForm").addEventListener("submit", this._onUpload);
    document.querySelector("#download").addEventListener("click", this._onDownload);
  }

  async _onUpload(event) {
    event.preventDefault();
    let input = document.querySelector("#fileInput");
    let data;
    try {
      data = await this._readFile(input);
    } catch (e) {
      alert(e.message);
      return;
    }
    if (!data) {
      alert("No file to upload");
      return;
    }

    let res = await fetch("/api/upload", {
      method: "POST",
      headers: { "Content-Type": "application/json" },
      body: JSON.stringify(data)
    });
    let json = await res.json();
    if (res.status !== 200) {
      alert(json.error);
      return;
    }
    input.value = "";
    alert("File uploaded");
  }

  async _onDownload(event) {
    let res = await fetch("/api/download");
    let json = await res.json();
    if (res.status !== 200) {
      alert(json.error);
      return;
    }
    let filename = json.filename;
    let content = json.content;
    this._startDownload(filename, content);
  }

  /* Read the selected file. input is the <input type="file"> element.
     Returns an object with a filename and content, or null if no file is selected.
     Throws an exception if reading the file fails (e.g. the file no longer exists on the user's computer). */
  async _readFile(input) {
    let file = input.files[0];
    if (!file) return null;
    /* Turn these callsbacks into a Promise so this function can be "await"ed. */
    let content = await new Promise((resolve, reject) => {
      let reader = new FileReader();
      reader.addEventListener("error", (event) => {
        reject(new Error("Error reading uploaded file"));
      });
      reader.addEventListener("load", (event) => {
        resolve(reader.result);
      });
      reader.readAsDataURL(file);
    });
    return { filename: file.name, content };
  }

  /* Start downloading filename with the given content (in the form of a data URL).
     This code creates an <a> element, clicks it, and removes it. You can also use the data URL as
     the href of a visible link, the src of an img, etc. */
  _startDownload(filename, content) {
    let link = document.createElement("a");
    link.style.display = "none";
    link.href = content;
    link.download = filename;
    document.body.append(link);
    link.click();
    link.remove();
  }
}
new App();
